package net.servodata.beei;

import net.servodata.beei.data.Correlatable;
import org.switchyard.*;
import org.switchyard.component.bpm.BPMConstants;
import org.switchyard.component.bpm.transaction.AS7JtaPlatform;
import org.switchyard.component.bpm.transaction.AS7TransactionHelper;

import javax.inject.Named;
import javax.transaction.InvalidTransactionException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import java.util.Arrays;
import java.util.List;

/**
 * @author Servodata a.s. <www.servodata.net>
 */
@Named
public class TransactionInterceptor implements ExchangeInterceptor {

	public static final String SUSPENDED_TRANSACTION_PNAME = "beei.cf.suspendedTransaction";

	@Override
	public void before(String target, Exchange exchange) throws HandlerException {
		if (exchange.getProvider() != null && "SimplePromotedService".equals(exchange.getProvider().getName().getLocalPart())) {
			try {
				Transaction t = AS7TransactionHelper.getTransactionManager().suspend();
				exchange.getContext().setProperty("suspendedTransaction", t, Scope.EXCHANGE);
				System.err.println("Suspended: " + t);

				printProperties("Before", exchange);
			} catch (SystemException e) {
				System.err.println(e);
			}
		}
	}

	@Override
	public void after(String target, Exchange exchange) throws HandlerException {
		if (exchange.getProvider() != null && "SimplePromotedService".equals(exchange.getProvider().getName().getLocalPart())) {
			
			printProperties("After", exchange);

			Transaction t = (Transaction) exchange.getContext().getProperty(SUSPENDED_TRANSACTION_PNAME, Scope.EXCHANGE).getValue();
			if (t != null) {
				try {
					AS7TransactionHelper.getTransactionManager().resume(t);
				} catch (InvalidTransactionException | SystemException e) {
					System.err.println(e);
				}
			} else {
				System.err.println(SUSPENDED_TRANSACTION_PNAME + " is null");
			}
		}
	}

	@Override
	public List<String> getTargets() {
		return Arrays.asList(PROVIDER);
	}

	private void printProperties(String s, Exchange exchange) {
		System.err.println(s);
		for (Property property : exchange.getContext().getProperties()) {
			if (property.getScope() == Scope.EXCHANGE) {
				System.out.println("\t" + property.getName() + ": " + property.getValue());
			}
		}
	}
}
